import com.aspose.tasks.*;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


class CustomWBSArranger implements ITreeAlgorithm {
    // level indexes
    private List levelCounters = new ArrayList();

    // separates codes
    private char separator = ',';

    /// <summary>
/// Converts integer to Roman number string.
/// </summary>
/// <param name="num"></param>
/// <returns></returns>
    private static String romanUCNumbering(int num) {
        StringBuilder sb = new StringBuilder();
        if (num >= 1000) {
            char[] chars = new char[num / 1000];
            Arrays.fill(chars, 'M');
            sb.append(chars);
            num %= 1000;
        }
        if (num >= 900) {
            sb.append("CM");
            num -= 900;
        }
        if (num >= 500) {
            sb.append('D');
            num -= 500;
        }
        if (num >= 400) {
            sb.append("CD");
            num -= 400;
        }
        if (num >= 100) {
            char[] chars = new char[num / 100];
            Arrays.fill(chars, 'C');
            sb.append(chars);
            num %= 100;
        }
        if (num >= 90) {
            sb.append("XC");
            num -= 90;
        }
        if (num >= 50) {
            sb.append('L');
            num -= 50;
        }
        if (num >= 40) {
            sb.append("XL");
            num -= 40;
        }
        if (num >= 10) {
            char[] chars = new char[num / 10];
            Arrays.fill(chars, 'X');
            sb.append(chars);
            num %= 10;
        }
        if (num >= 9) {
            sb.append("IX");
            num -= 9;
        }
        if (num >= 5) {
            sb.append('V');
            num -= 5;
        }
        if (num >= 4) {
            sb.append("IV");
            num -= 4;
        }
        if (num >= 1) {
            char[] chars = new char[num];
            Arrays.fill(chars, 'I');
            sb.append(chars);
        }
        return sb.toString();
    }

    public CustomWBSArranger() {
        // init level indexes
        levelCounters.clear();
        for (int i = 0; i < 10; i++) {
            levelCounters.add(0);
        }
    }

    /// <summary>
/// This method increments current level counter in the list of level counters
/// </summary>
/// <param name="level"></param>
    private void incLevelCounter(int level) {
        if (level > levelCounters.size()) {
            return;
        } else {
            levelCounters.set(level - 1,((Integer) levelCounters.get(level - 1) + 1));
        }
    }

    public void preAlg(Object el, int level) {
    }

    /// <summary>
///
/// </summary>
/// <param name="el"></param>
/// <param name="level"></param>
    public void alg(Object el, int level) {
        // skips zero level
        if (level == 0) {
            return;
        }
        Task task = (Task) el;
        String wbs = "";
        incLevelCounter(level);
        for (int i = 0; i < level; i++) {
            // compose wbs code
            if (i > 0) {
                wbs += separator;
            }
            wbs += romanUCNumbering((Integer) levelCounters.get(i));
        }
        task.setWbs(wbs);
    }

    public void postAlg(Object el, int level) {
        if (level < levelCounters.size())
            levelCounters.set(level, 0);
    }
}

class ArrangeWBS {
    /// <summary>
    /// The main entry point for the application.
    /// </summary>
    public static void main(String[] args) {
        try {
            if (args.length != 0) {
                System.out.println("Usage: Aspose.Tasks.ArrangeWBS <input file name>");
            } else {
                //C:\Work\Projects\aspose\Aspose.Tasks.Java\trunk\external\src\test\resources\com\aspose\tasks\WBS\DefaultWBS2003.mpp
                processWBS("C:\\Work\\Projects\\aspose\\Aspose.Tasks.Java\\trunk\\external\\src\\test\\resources\\com\\aspose\\tasks\\WBS\\DefaultWBS2003.mpp");
            }
        } catch (Exception ex) {
            System.out.println(ex.getStackTrace());
        }
    }

    private static void processWBS(String fileName) throws FileNotFoundException {
        // reads project from file
        ProjectReader reader = new ProjectReader();
        Project project = reader.read(new FileInputStream(fileName));

        // uses TaskUtils to apply CustomWBSArranger to tasks tree
        TaskUtils.apply(project.getRootTask(), new CustomWBSArranger(), 0);

        // writes project
        ProjectWriter writer = new ProjectWriter();
        writer.write(project, new FileOutputStream(fileName + "mod", false), TasksDataFormat.XML);
    }
}
