import com.aspose.tasks.*;

import java.io.FileInputStream;
import java.io.FileNotFoundException;


class CombineConditions {
    /// <summary>
    /// main method
    /// </summary>
    public static void main(String[] args) {
        try {
            if (args.length != 3) {
                System.out.println(
                        "Usage: Aspose.Tasks.CombineConditions <input file name> <taskName> <task id>");
            } else {
                filter(args[0], args[1], args[2]);
            }
        } catch (Exception ex) {
            System.out.println(ex.getStackTrace());
        }
    }

    private static void filter(String fileName, String taskName, String taskID) throws FileNotFoundException {
        // reads project from file
        ProjectReader reader = new ProjectReader();
        Project project = reader.read(new FileInputStream(fileName));

        // find task
        Task task = TaskUtils.find(project.getRootTask(), new And(new NameCondition(taskName),
                new MoreThanIDCondition(Integer.parseInt(taskID))));

        // Prints task parameters
        if (task != null) {
            System.out.println("Name   : " + task.getName());
            System.out.println("UID    : " + task.getUid());
            System.out.println("ID     : " + task.getId());
            System.out.println("Start  : " + task.getStart());
            System.out.println("Finish : " + task.getFinish());
        }
    }

    /// <summary>
    /// Define find condition. Method check() of this class returns true if task name starts with the pattern otherwise returns false;
    /// </summary>
    private static class NameCondition implements ICondition {
        private String pattern;

        public NameCondition(String pattern) {
            this.pattern = pattern;
        }

        public boolean check(Object el) {
            Task task = (Task) el;
            if (task.getName() != null) {
                return task.getName().startsWith(pattern);
            }
            return false;
        }
    }

    private static class MoreThanIDCondition implements ICondition {
        private int id;

        public MoreThanIDCondition(int id) {
            this.id = id;
        }

        public boolean check(Object el) {
            Task task = (Task) el;
            return task.getId() > id;
        }
    }
}
