import com.aspose.tasks.*;

import java.io.FileInputStream;
import java.io.FileNotFoundException;

class FilterByName {
    /// <summary>
    /// main method
    /// </summary>
    public static void main(String[] args) {
        try {
            if (args.length != 2) {
                System.out.println("Usage: Aspose.Tasks.FilterByName <input file name> <taskName>");
            } else {
                filter(args[0], args[1]);
            }
        } catch (Exception ex) {
            System.out.println(ex.getStackTrace());
        }
    }

    private static void filter(String fileName, String pattern) throws FileNotFoundException {
        // reads project from file
        ProjectReader reader = new ProjectReader();
        Project project = reader.read(new FileInputStream(fileName));

        // filter tasks
        Task newRoot = TaskUtils.filter(project.getRootTask(), new NameCondition(pattern));
        // print tasks
        TaskUtils.apply(newRoot, new TreePrinter(), 0);
    }
}

class TreePrinter implements ITreeAlgorithm {
    private String indent = "";

    /// <summary>
/// Increase indent
/// </summary>
/// <param name="el"></param>
/// <param name="index"></param>
    public void preAlg(Object el, int index) {
        indent = indent + " ";
    }


    public void alg(Object el, int index) {
        Task task = (Task) el;
        System.out.println(task.getUid() + " " + indent + task.getName());
    }

    /// <summary>
/// Decrease indent
/// </summary>
/// <param name="el"></param>
/// <param name="index"></param>
    public void postAlg(Object el, int index) {
        indent = indent.substring(0, indent.length() - 1);
    }
}

/// <summary>
/// Define find condition. Method Check() of this class returns true if task name starts with the pattern otherwise returns false;
/// </summary>
class NameCondition implements ICondition {
    private String pattern;

    public NameCondition(String pattern) {
        this.pattern = pattern;
    }

    public boolean check(Object el) {
        Task task = (Task) el;
        if (task.getName() != null) {
            return task.getName().startsWith(pattern);
        }
        return false;
    }
}